/*
 * Copyright (c) 2021 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.eac;

/**
 * Interface to a Terminal Authentication Signer
 *
 * This interface is modeled as per BSI TR-03129.
 *
 * In difference to TR-03129 the raw signature input is provided rather than
 * the hash value.
 *
 * @author asc
 */
public interface TerminalAuthenticationSigner {

	/**
	 * Return a list of Card Verifiable Certificates, starting with the certificate that can
	 * verified by the public key referenced in parameter keyCAR and ending with the certificate
	 * that is issued to the terminal authentication signer in use.
	 *
	 * @param keyCAR the Certificate Holder Reference (CAR) of the public key in the card
	 * that can be directly used to verify a certificate. This is typically a root or link
	 * certificate or a DVCA certificate that was already verified by the card.
	 *
	 * @return the list of CVCs.
	 */
	public byte[][] getCertificateChain(String keyCAR);



	/**
	 * Calculate a signature over the data to be signed using the terminal authentication
	 * key identified by keyCHR. The signature must be return in a format suitable for
	 * presentation to the card, i.e. in R|S format for ECDSA.
	 *
	 * The hash algorithm is determined by the terminal authentication signer.
	 *
	 * @param dataTBS the input for the hash and the signature operation
	 * @param keyCHR the terminal authentication key
	 *
	 * @return the signature
	 */
	public byte[] getTASignature(byte[] dataTBS, String keyCHR);
}
