/*
 * Copyright (c) 2021 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.eac.cvc;

import de.cardcontact.tlv.ConstructedTLV;
import de.cardcontact.tlv.TLV;
import de.cardcontact.tlv.TLVEncodingException;
import de.cardcontact.tlv.Tag;

public class AuthenticatedRequest extends ConstructedTLV {

	public final static Tag TAG = new Tag(7, Tag.APPLICATION, true);
	final static String NAME = "Authentication";


	public AuthenticatedRequest(CVCertificate cert, CertificationAuthorityReference car, SignatureTLV signature) {
		super(TAG);
		setName(NAME);
		add(cert);
		add(car);
		add(signature);
	}



	public AuthenticatedRequest(TLV tlv) throws TLVEncodingException {
		super(tlv);
		setName(NAME);

		if (!tlv.getTag().equals(TAG)) {
			throw new TLVEncodingException(NAME + " with wrong tag " + tlv.getTag());
		}

		if (childs.size() != 3) {
			throw new TLVEncodingException(NAME + " must contain 3 elements");
		}

		childs.set(0, new CVCertificate(childs.get(0)));
		childs.set(1, new CertificationAuthorityReference(childs.get(1)));
		childs.set(2, new SignatureTLV(childs.get(2)));
	}



	public CVCertificate getCVCertificate() {
		return (CVCertificate)childs.get(0);
	}



	public CertificationAuthorityReference getCertificationAuthorityReference() {
		return (CertificationAuthorityReference)childs.get(1);
	}



	public SignatureTLV getSignatureTLV() {
		return (SignatureTLV)childs.get(2);
	}

}
