package de.cardcontact.opencard.eac.cvc;

import java.security.GeneralSecurityException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.interfaces.ECPrivateKey;
import java.security.interfaces.ECPublicKey;

import de.cardcontact.tlv.ObjectIdentifier;

public class CVCertificateRequestGenerator {

	private CertificationAuthorityReference car;
	private CertificateHolderReference chr;
	private ObjectIdentifier pubalgo = ECPublicKeyTLV.id_TA_ECDSA_SHA_256;
	private Extensions extensions = null;
	private PublicKey pub;



	public CVCertificateRequestGenerator() {

	}



	public void setCertificationAuthorityReference(CertificationAuthorityReference car) {
		this.car = car;
	}



	public void setPublicKey(PublicKey pub) {
		this.pub = pub;
	}



	public void setPublicKey(PublicKey pub, ObjectIdentifier algo) {
		this.pub = pub;
		this.pubalgo = algo;
	}



	public void setCertificateHolderReference(CertificateHolderReference chr) {
		this.chr = chr;
	}



	public void addExtension(Extension extension) {
		if (this.extensions == null) {
			this.extensions = new Extensions();
		}
		this.extensions.add(extension);
	}



	public CVCertificate generate(PrivateKey signerkey, String algo) throws GeneralSecurityException {
		PublicKeyTLV pt = new ECPublicKeyTLV(this.pubalgo, (ECPublicKey)this.pub, true);
		CertificateBody body = new CertificateBody(car, pt, chr);

		if (this.extensions != null) {
			body.add(this.extensions);
		}

		Signature signer = Signature.getInstance(algo);
		signer.initSign(signerkey);
		signer.update(body.getBytes());
		byte[] signature = signer.sign();

		if (signerkey instanceof ECPrivateKey) {
			int keysize = ((ECPrivateKey)signerkey).getParams().getCurve().getField().getFieldSize();
			keysize = (keysize + 7) >> 3;
			signature = ECSignature.unwrapSignature(signature, keysize);
		}
		return new CVCertificate(body, new SignatureTLV(signature));
	}



	public CVCertificate generate(PrivateKey signerkey) throws GeneralSecurityException {
		return generate(signerkey, "SHA256withECDSA");
	}



	public AuthenticatedRequest authenticate(CVCertificate request, PrivateKey authenticationKey, String algo, CertificationAuthorityReference car) throws GeneralSecurityException {
		Signature signer = Signature.getInstance(algo);
		signer.initSign(authenticationKey);
		signer.update(request.getBytes());
		signer.update(car.getBytes());
		byte[] signature = signer.sign();

		if (authenticationKey instanceof ECPrivateKey) {
			int keysize = ((ECPrivateKey)authenticationKey).getParams().getCurve().getField().getFieldSize();
			keysize = (keysize + 7) >> 3;
			signature = ECSignature.unwrapSignature(signature, keysize);
		}

		return new AuthenticatedRequest(request, car, new SignatureTLV(signature));
	}
}
