/*
 * Copyright (c) 2021 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.eac.cvc;

import de.cardcontact.tlv.ConstructedTLV;
import de.cardcontact.tlv.ObjectIdentifier;
import de.cardcontact.tlv.PrimitiveTLV;
import de.cardcontact.tlv.TLV;
import de.cardcontact.tlv.TLVEncodingException;
import de.cardcontact.tlv.Tag;

public class CertificateHolderAuthorizationTemplate extends ConstructedTLV {

	final static Tag TAG = new Tag(76, Tag.APPLICATION, true);
	final static String NAME = "Certificate Holder Authorization Template";
	final static Tag TAG_RA = new Tag(19, Tag.APPLICATION, false);

	public final static ObjectIdentifier ID_IS_OID = new ObjectIdentifier(new int[] { 0,4,0,127,0,7,3,1,2,1 });
	public final static ObjectIdentifier ID_AT_OID = new ObjectIdentifier(new int[] { 0,4,0,127,0,7,3,1,2,2 });
	public final static ObjectIdentifier ID_ST_OID = new ObjectIdentifier(new int[] { 0,4,0,127,0,7,3,1,2,3 });
	public final static ObjectIdentifier ID_CC_ROLE_SC_HSM = new ObjectIdentifier(new int[] { 1,3,6,1,4,1,24991,3,1,1 });



	public CertificateHolderAuthorizationTemplate(ObjectIdentifier oid, byte[] relativeAuthorization) {
		super(TAG);
		setName(NAME);
		add(oid);
		add(new PrimitiveTLV(TAG_RA, relativeAuthorization));
	}



	public CertificateHolderAuthorizationTemplate(TLV tlv) throws TLVEncodingException {
		super(tlv);
		setName(NAME);

		if (!tlv.getTag().equals(TAG)) {
			throw new TLVEncodingException(NAME + " with wrong tag " + tlv.getTag());
		}

		if (childs.size() != 2) {
			throw new TLVEncodingException(NAME + " must contain 2 elements");
		}

		TLV t = childs.get(0);
		if (!(t instanceof ObjectIdentifier)) {
			throw new TLVEncodingException(NAME + " first element must be an Object Identifier");
		}

		t = childs.get(1);
		if (!t.getTag().equals(TAG_RA)) {
			throw new TLVEncodingException("Relative authorization in " + NAME + " with wrong tag " + tlv.getTag());
		}
	}

}
