/*
 * Copyright (c) 2021 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.eac.cvc;

import de.cardcontact.tlv.ConstructedTLV;
import de.cardcontact.tlv.ObjectIdentifier;
import de.cardcontact.tlv.TLV;
import de.cardcontact.tlv.TLVEncodingException;
import de.cardcontact.tlv.Tag;

public class Extension extends ConstructedTLV {

	final static Tag TAG = new Tag(19, Tag.APPLICATION, true);
	final static String NAME = "Extension";



	public Extension(ObjectIdentifier oid) {
		super(TAG);
		setName(NAME);
		add(oid);
	}



	public Extension(TLV tlv) throws TLVEncodingException {
		super(tlv);
		if (!tlv.getTag().equals(TAG)) {
			throw new TLVEncodingException(NAME + " with wrong tag " + tlv.getTag());
		}

		setName(NAME);

		if (childs.size() < 2) {
			throw new TLVEncodingException(NAME + " must contain at least 2 elements");
		}

		if (!(childs.get(0) instanceof ObjectIdentifier)) {
			throw new TLVEncodingException(NAME + " must start with Object Identifier");
		}
	}



	public static Extension newExtension(TLV tlv) throws TLVEncodingException {
		if (!tlv.getTag().equals(TAG)) {
			throw new TLVEncodingException(NAME + " with wrong tag " + tlv.getTag());
		}

		ConstructedTLV ctlv = (ConstructedTLV)tlv;
		if (ctlv.getChildCount() < 1) {
			throw new TLVEncodingException(NAME + " must contain at least 1 element");
		}

		if (!(ctlv.getChildAt(0) instanceof ObjectIdentifier)) {
			throw new TLVEncodingException(NAME + " must start with Object Identifier");
		}

		ObjectIdentifier oid = (ObjectIdentifier)ctlv.getChildAt(0);

		if (oid.equals(KeyDomainUIDExtension.id_key_domain_uid)) {
			return new KeyDomainUIDExtension(tlv);
		}
		return new Extension(tlv);
	}
}
