/*
 * Copyright (c) 2021 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.eac.pkcs8;

import java.math.BigInteger;
import java.security.spec.ECField;
import java.security.spec.ECParameterSpec;
import java.security.spec.ECPoint;
import java.security.spec.EllipticCurve;

import de.cardcontact.opencard.eac.cvc.ECPublicKeyTLV;
import de.cardcontact.tlv.IntegerTLV;
import de.cardcontact.tlv.ObjectIdentifier;
import de.cardcontact.tlv.OctetString;
import de.cardcontact.tlv.Sequence;
import de.cardcontact.tlv.TLV;
import de.cardcontact.tlv.TLVEncodingException;

public class ECParameters extends Sequence {

	public final static ObjectIdentifier ID_ECPUBLICKEY = new ObjectIdentifier(new int[] { 1,2,840,10045,2,1 });
	final static String NAME = "ECParameters";

	public ECParameters(TLV tlv) throws TLVEncodingException {
		super(tlv);

		setName(NAME);

		if (childs.size() != 5) {
			throw new TLVEncodingException(NAME + " must contain 5 elements");
		}

		if (!(childs.get(0) instanceof IntegerTLV)) {
			throw new TLVEncodingException("Version must be INTEGER");
		}

		IntegerTLV itlv = (IntegerTLV)childs.get(0);
		if (itlv.getLong() != 1) {
			throw new TLVEncodingException("Version must be 1");
		}

		childs.set(1, new FieldID(childs.get(1)));
		childs.set(2, new Curve(childs.get(2)));

		if (!(childs.get(3) instanceof OctetString)) {
			throw new TLVEncodingException("Base point must be OCTET-STRING");
		}

		if (!(childs.get(4) instanceof IntegerTLV)) {
			throw new TLVEncodingException("Order must be INTEGER");
		}
	}



	public ECParameterSpec getECParameterSpec() {
		ECField field = ((FieldID)childs.get(1)).getField();
		EllipticCurve curve = ((Curve)childs.get(2)).getCurve(field);

		ECPoint g = ECPublicKeyTLV.decodePoint(childs.get(3).getValue());
		BigInteger r = ((IntegerTLV)childs.get(4)).getBigInteger();
		return new ECParameterSpec(curve, g, r, 1);
	}
}
