/*
 * Copyright (c) 2016 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.service.isocard;

import opencard.core.terminal.CommandAPDU;

public class IsoCommandAPDU extends CommandAPDU {

	public IsoCommandAPDU(byte cla, byte ins, byte p1, byte p2) {
		this(cla, ins, p1, p2, null, -1);
	}



	public IsoCommandAPDU(byte cla, byte ins, byte p1, byte p2, byte[] cdata) {
		this(cla, ins, p1, p2, cdata, 0);
	}



	public IsoCommandAPDU(byte cla, byte ins, byte p1, byte p2, byte[] cdata, int Ne) {
		super(9 + (cdata != null ? cdata.length : 0));

		this.apdu_buffer[0] = cla;
		this.apdu_buffer[1] = ins;
		this.apdu_buffer[2] = p1;
		this.apdu_buffer[3] = p2;
		this.apdu_length  = 4;

		boolean extended = false;
		if (cdata != null) {
			if ((cdata.length > 255) || (Ne > 256)) {
				this.apdu_buffer[this.apdu_length++] = (byte)0;
				this.apdu_buffer[this.apdu_length++] = (byte)(cdata.length >> 8);
				extended = true;
			}
			this.apdu_buffer[this.apdu_length++] = (byte)(cdata.length & 0xFF);
			System.arraycopy(cdata, 0, this.apdu_buffer, this.apdu_length, cdata.length);
			this.apdu_length += cdata.length;
		}

		if (Ne > 0) {
			if ((Ne <= 256) && !extended) {
				this.apdu_buffer[this.apdu_length++] = (byte)(Ne & 0xFF);
			} else {
				this.apdu_buffer[this.apdu_length++] = (byte)((Ne >> 8) & 0xFF);
				this.apdu_buffer[this.apdu_length++] = (byte)(Ne & 0xFF);
			}
		}
	}
}
