/*
 * Copyright (c) 2021 CardContact Systems GmbH, Minden, Germany.
 *
 * Redistribution and use in source (source code) and binary (object code)
 * forms, with or without modification, are permitted provided that the
 * following conditions are met:
 * 1. Redistributed source code must retain the above copyright notice, this
 * list of conditions and the disclaimer below.
 * 2. Redistributed object code must reproduce the above copyright notice,
 * this list of conditions and the disclaimer below in the documentation
 * and/or other materials provided with the distribution.
 * 3. The name of CardContact may not be used to endorse or promote products derived
 * from this software or in any other form without specific prior written
 * permission from CardContact.
 * 4. Redistribution of any modified code must be labeled "Code derived from
 * the original OpenCard Framework".
 *
 * THIS SOFTWARE IS PROVIDED BY CardContact "AS IS" FREE OF CHARGE. CardContact SHALL NOT BE
 * LIABLE FOR INFRINGEMENTS OF THIRD PARTIES RIGHTS BASED ON THIS SOFTWARE.  ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  CardContact DOES NOT WARRANT THAT THE FUNCTIONS CONTAINED IN THIS
 * SOFTWARE WILL MEET THE USER'S REQUIREMENTS OR THAT THE OPERATION OF IT WILL
 * BE UNINTERRUPTED OR ERROR-FREE.  IN NO EVENT, UNLESS REQUIRED BY APPLICABLE
 * LAW, SHALL CardContact BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  ALSO, CardContact IS UNDER NO OBLIGATION
 * TO MAINTAIN, CORRECT, UPDATE, CHANGE, MODIFY, OR OTHERWISE SUPPORT THIS
 * SOFTWARE.
 */

package de.cardcontact.opencard.service.isocard.apdu;

import de.cardcontact.tlv.ConstructedTLV;
import de.cardcontact.tlv.TLV;
import de.cardcontact.tlv.TLVEncodingException;
import de.cardcontact.tlv.Tag;

public class ChipAuthenticationResponseData extends ConstructedTLV {

	final static Tag TAG_NONCE = new Tag(1, Tag.CONTEXT, false);
	final static Tag TAG_TOKEN = new Tag(2, Tag.CONTEXT, false);

	public ChipAuthenticationResponseData(TLV tlv) throws TLVEncodingException {
		super(tlv);

		if (!tlv.getTag().equals(GeneralAuthenticateCommandAPDU.TAG_DAD)) {
			throw new TLVEncodingException("Dynamic Authentication Data with wrong tag " + tlv.getTag());
		}

		if (childs.size() != 2) {
			throw new TLVEncodingException("Dynamic Authentication Data must contain two elements");
		}

		TLV t = childs.get(0);
		if (!t.getTag().equals(TAG_NONCE) || t.getLength() != 8) {
			throw new TLVEncodingException("Dynamic Authentication Data contains invalid nonce");
		}

		t = childs.get(1);
		if (!t.getTag().equals(TAG_TOKEN) || t.getLength() != 8) {
			throw new TLVEncodingException("Dynamic Authentication Data contains invalid authentication token");
		}
	}



	public byte[] getNonce() {
		return childs.get(0).getValue();
	}



	public byte[] getAuthenticationToken() {
		return childs.get(1).getValue();
	}



	public static ChipAuthenticationResponseData parse(byte[] rsp) throws TLVEncodingException {
		TLV tlv = TLV.factory(rsp);
		return new ChipAuthenticationResponseData(tlv);
	}
}
