package opencard.core.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.cardcontact.tlv.HexString;
import opencard.core.terminal.CardID;
import opencard.core.terminal.CommandAPDU;
import opencard.core.terminal.ResponseAPDU;
import opencard.core.terminal.SlotChannel;

public class SLF4JAPDUTracer implements APDUTracer {

	final Logger logger = LoggerFactory.getLogger(SLF4JAPDUTracer.class);



	@Override
	public void traceCommandAPDU(SlotChannel sc, CommandAPDU capdu) {
		if (logger.isDebugEnabled()) {
			int slotId = (sc.getCardTerminal().getName().hashCode() + sc.getSlotNumber()) & 0xFF;

			String s = HexString.hexifyByte(slotId);
			s = s.concat(" ");
			s = s.concat(APDUFormatter.commandAPDUToString(capdu));
			logger.debug(s);
		}
	}



	@Override
	public void traceResponseAPDU(SlotChannel sc, ResponseAPDU rapdu) {
		if (logger.isDebugEnabled()) {
			String s = APDUFormatter.responseAPDUToString(rapdu);
			logger.debug(s);
		}
	}



	@Override
	public void traceAnswerToReset(SlotChannel sc, CardID cardID) {
		if (logger.isDebugEnabled()) {
			logger.debug(" ATR: " + HexString.hexifyByteArray(cardID.getATR()));
		}
	}

}
